/*
 * Trackball code:
 *
 * Implementation of a virtual trackball.
 * Implemented by Gavin Bell, lots of ideas from Thant Tessman and
 *   the August '88 issue of Siggraph's "Computer Graphics," pp. 121-129.
 *
 * Vector manip code:
 *
 * Original code from:
 * David M. Ciemiewicz, Mark Grossman, Henry Moreton, and Paul Haeberli
 *
 * Much mucking with by:
 * Gavin Bell
 */

// TrackBall.h: interface for the TrackBall class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TRACKBALL_H__33E455C1_98C0_11D1_982A_00A0C98A5C22__INCLUDED_)
#define AFX_TRACKBALL_H__33E455C1_98C0_11D1_982A_00A0C98A5C22__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <GL\gl.h>

class TrackBall  
{
public:
	void init();
	TrackBall();
	virtual ~TrackBall() {};
	/*
	 * Pass the x and y coordinates of the last and current positions of
	 * the mouse, scaled so they are from (-1.0 ... 1.0).
	 *
	 * The resulting rotation is returned as a quaternion rotation in the
	 * first paramater.
	 */

	TrackBall(float sx, float sy, float ex, float ey);

	void setTrackBallSize(float size);

	/*
	 * Given two quaternions, add them together to get a third quaternion.
	 * Adding quaternions to get a compound rotation is analagous to adding
	 * translations to get a compound translation.  When incrementally
	 * adding rotations, the first argument here should be the new
	 * rotation, the second and third the total rotation (which will be
	 * over-written with the resulting new total rotation).
	 */
	void add_quats(float deltaQuat[4]);

	/*
	 * A useful function, builds a rotation matrix in Matrix based on
	 * given quaternion.
	 */
	float *return_quats() const;
	GLfloat *return_matrix();

	void getMatrix( float [][4] );

private:
	float	_currentQuat[4];
	float	_trackBallSize;			// track ball size 
	GLfloat _m[4][4];

	void vzero(float *v);
	void vset(float *v, float x, float y, float z);
	void vadd(const float *src1, const float *src2, float *dst);
	void vsub(const float *src1, const float *src2, float *dst);
	void vcopy(const float *v1, float *v2);
	void vcross(const float *v1, const float *v2, float *cross);
	float vlength(const float *v);
	void vscale(float *v, float div);
	void vnormal(float *v);
	float vdot(const float *v1, const float *v2);
	float tb_project_to_sphere(float r, float x, float y);
	void normalize_quat(float q[4]);
	void build_rotmatrix();

	/*
	 * This function computes a quaternion based on an axis (defined by
	 * the given vector) and an angle about which to rotate.  The angle is
	 * expressed in radians.  The result is put into the third argument.
	 */
	void axis_to_quat(float a[3], float phi, float q[4]);
};

#endif // !defined(AFX_TRACKBALL_H__33E455C1_98C0_11D1_982A_00A0C98A5C22__INCLUDED_)
